(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.parsePathLists = exports.parsePath = void 0;
    var viewModelParseWatchCache = {};
    function trimRegEx(t, r) {
        var m = r.exec(t);
        if (m && m.length) {
            return [true, t.substring(m[0].length).trim(), m[0]];
        }
        return [false, t, ""];
    }
    function extractTill(text, search, returnOriginal) {
        if (returnOriginal === void 0) { returnOriginal = false; }
        var index = text.indexOf(search);
        if (index <= 0) {
            return returnOriginal ? text : "";
        }
        return text.substring(0, index);
    }
    function parsePath(f, parseThis) {
        var _a;
        var str = f.toString().trim();
        str = str.split("\n").filter(function (s) { return !/^\/\//.test(s.trim()); }).join("\n");
        var key = (parseThis === undefined ? "un:" : (parseThis ? "_this:" : "_noThis:")) + str;
        var px1 = viewModelParseWatchCache[key];
        if (px1) {
            return px1;
        }
        if (str.endsWith("}")) {
            str = str.substr(0, str.length - 1);
        }
        var functionRegEx = /^\w+\s*\(/;
        var lambdaRegEx = /^\(?(\w+)?(\s?\,\s?\w+)*\)?\s?\=\>/;
        var arg = "";
        var original = str;
        // tslint:disable-next-line: prefer-const
        var _b = trimRegEx(str, functionRegEx), success = _b[0], remaining = _b[1];
        if (success) {
            str = remaining;
            remaining = extractTill(remaining, ")");
            str = str.substring(remaining.length);
            arg = extractTill(remaining, ",", true);
        }
        else {
            // this must be a lambda..
            _a = trimRegEx(str, lambdaRegEx), success = _a[0], str = _a[1], remaining = _a[2];
            if (success) {
                remaining = remaining.trim();
                remaining = remaining.substring(0, remaining.length - 2);
                remaining = extractTill(remaining, ")", true);
                arg = extractTill(remaining, ",", true);
                if (arg.startsWith("(")) {
                    arg = arg.substring(1);
                }
            }
            else {
                if (parseThis !== undefined && parseThis === false) {
                    return [];
                }
                else {
                    parseThis = true;
                }
            }
        }
        // if (str.startsWith("function (")) {
        //     str = str.substr("function (".length);
        // } else if (str.startsWith("function(")) {
        //     str = str.substr("function(".length);
        // } else {
        //     const sb = str.indexOf("(");
        //     if (sb !== -1) {
        //         str = str.substr(sb + 1);
        //     } else {
        //         if (parseThis !== undefined && parseThis === false) {
        //             return [];
        //         } else {
        //             parseThis = true;
        //         }
        //     }
        // }
        str = str.trim();
        // const commaIndex = str.indexOf(",");
        // if (commaIndex !== -1 && commaIndex < index) {
        //     index = commaIndex;
        // }
        // const lambdaIndex = str.indexOf("=>");
        // if (lambdaIndex !== -1) {
        //     index = lambdaIndex;
        // }
        var isThis = parseThis === undefined
            ? ((arg ? false : true) || parseThis)
            : parseThis;
        var p = (isThis ? "(\\_this|this)" : (arg || "")).trim();
        /**
         * This is the case when there is no parameter to check and there `parseThis` is false
         */
        if (p.length === 0) {
            var empty = [];
            viewModelParseWatchCache[key] = empty;
            return empty;
        }
        // str = str.substr(index + 1);
        var regExp = "(?:(\\b" + p + ")(?:(\\.[a-zA-Z_][a-zA-Z_0-9]*)+)\\s?(?:(\\(|\\=\\=\\=|\\=\\=|\\=)?))";
        var re = new RegExp(regExp, "gi");
        var path = [];
        var ms = str.replace(re, function (m) {
            // console.log(`m: ${m}`);
            var px = m;
            if (px.startsWith("this.")) {
                if (parseThis !== true) {
                    px = px.substr(5);
                }
            }
            else if (px.startsWith("_this.")) {
                if (parseThis !== true) {
                    px = px.substr(6);
                }
                else {
                    // need to convert _this to this
                    px = px.substr(1);
                }
            }
            else {
                px = px.substr(p.length + 1);
            }
            px = px.split(".").filter(function (s) { return !s.endsWith("("); }).join(".");
            // console.log(px);
            if (!path.find(function (y) { return y === px; })) {
                path.push(px);
            }
            // path = path.filter( (f1) => f1.endsWith("==") || !(f1.endsWith("(") || f1.endsWith("=") ));
            // path = path.map(
            //     (px2) => (px2.endsWith("===") ? px2.substr(0, px2.length - 3) :
            //         ( px2.endsWith("==") ? px2.substr(0, px2.length - 2) : px2 )).trim() );
            var filtered = [];
            for (var _i = 0, path_2 = path; _i < path_2.length; _i++) {
                var iterator = path_2[_i];
                if (iterator.endsWith("==") || !(iterator.endsWith("(") || iterator.endsWith("="))) {
                    filtered.push((iterator.endsWith("===") ? iterator.substr(0, iterator.length - 3) :
                        (iterator.endsWith("==") ? iterator.substr(0, iterator.length - 2) : iterator)).trim());
                }
            }
            path = filtered.filter(function (px11) {
                var search = px11 + ".";
                for (var _i = 0, filtered_1 = filtered; _i < filtered_1.length; _i++) {
                    var iterator = filtered_1[_i];
                    if (px11 !== iterator && iterator.indexOf(search) !== -1) {
                        return false;
                    }
                }
                return true;
            });
            return m;
        });
        path = path.sort(function (a, b) { return b.localeCompare(a); });
        var duplicates = path;
        path = [];
        var _loop_1 = function (iterator) {
            if (path.find(function (px2) { return px2 === iterator; })) {
                return "continue";
            }
            path.push(iterator);
        };
        for (var _i = 0, duplicates_1 = duplicates; _i < duplicates_1.length; _i++) {
            var iterator = duplicates_1[_i];
            _loop_1(iterator);
        }
        var rp = [];
        var _loop_2 = function (rpItem) {
            if (rp.find(function (x) { return x.startsWith(rpItem); })) {
                return "continue";
            }
            rp.push(rpItem);
        };
        for (var _c = 0, path_1 = path; _c < path_1.length; _c++) {
            var rpItem = path_1[_c];
            _loop_2(rpItem);
        }
        // tslint:disable-next-line: no-console
        // console.log(`Watching: ${path.join(", ")}`);
        var pl = path.filter(function (p1) { return p1; }).map(function (p1) { return p1.split("."); });
        viewModelParseWatchCache[key] = pl;
        return pl;
    }
    exports.parsePath = parsePath;
    var viewModelParseWatchCache2 = {};
    function parsePathLists(f) {
        var str = f.toString().trim();
        var key = str;
        var px1 = viewModelParseWatchCache2[key];
        if (px1) {
            return px1;
        }
        // str = str.split("\n").filter((s) => !/^\/\//.test(s.trim())).join("\n");
        // if (str.endsWith("}")) {
        //     str = str.substr(0, str.length - 1);
        // }
        // if (str.startsWith("function (")) {
        //     str = str.substr("function (".length);
        // }
        // if (str.startsWith("function(")) {
        //     str = str.substr("function(".length);
        // }
        // str = str.trim();
        var pl = {
            pathList: parsePath(str, false),
            thisPath: parsePath(str, true),
            combined: []
        };
        if (pl.thisPath.length && pl.pathList.length) {
            // we need to combine this
            // pl.combinedPathList =
            pl.combined = pl.thisPath
                .map(function (x) {
                x[0] = "t";
                x.splice(0, 0, "this");
                return x;
            })
                .concat(pl.pathList.map(function (x) {
                x.splice(0, 0, "this", "x");
                return x;
            }));
            pl.thisPath = [];
            pl.pathList = [];
        }
        viewModelParseWatchCache2[key] = pl;
        return pl;
    }
    exports.parsePathLists = parsePathLists;
});
//# sourceMappingURL=ExpressionParser.js.map